//  This file is part of ff3d - http://www.freefem.org/ff3d
//  Copyright (C) 2007 Driss Yakoubi

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2, or (at your option)
//  any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

//  $Id: LagrangeBasis.cpp,v 1.2 2008/01/23 23:30:53 delpinux Exp $

#include <LagrangeBasis.hpp>
#include <ErrorHandler.hpp>

LagrangeBasis::
LagrangeBasis(const LagrangeBasis& lagrangeBasis)
  : __nodes(lagrangeBasis.__nodes),
    __denominatorValues(lagrangeBasis.__denominatorValues)
{
  ;
}

LagrangeBasis::
LagrangeBasis(const Vector<real_t>& nodes)
  : __nodes(nodes),
    __denominatorValues(__nodes.size()) 
{
  __denominatorValues = 1.;
  for(size_t i=0; i< __nodes.size(); ++i) {
    for(size_t j=0; j<i; ++j) {
      real_t xi_xj = __nodes[i]-__nodes[j];
      __denominatorValues[i] *=  xi_xj;
      __denominatorValues[j] *= -xi_xj;
    }
  }
  for(size_t i=0; i< __nodes.size(); ++i) {
    __denominatorValues[i] = 1./ __denominatorValues[i]; 
  }
}

LagrangeBasis::
~LagrangeBasis()
{
  ;
}

void LagrangeBasis::
getValues(const real_t& x,
	  Vector<real_t>& values) const
{
  values = __denominatorValues;
  if (__nodes.size()== 0) return ;
  real_t p=1;
  for(size_t i=1;i<__nodes.size();++i ){
    p *= x-__nodes[i-1];
    values[i] *= p;
  }

  real_t q=1;
  for(int i=__nodes.size()-2;i>=0;--i ){
    q *= x-__nodes[i+1];
    values[i] *= q;
  }
}

void  LagrangeBasis::
getDerivativeValues(const real_t& x,
		    Vector<real_t>& derivateValues) const
{
}

real_t LagrangeBasis::
getValue(const real_t& x, 
	 const size_t& i) const
{
  throw ErrorHandler(__FILE__,__LINE__,
		     "not implemented",
		     ErrorHandler::unexpected);
  return 0;
}

real_t LagrangeBasis::
getDerivativeValue(const real_t& x, 
		   const size_t& i) const
{
  throw ErrorHandler(__FILE__,__LINE__,
		     "not implemented",
		     ErrorHandler::unexpected);
  return 0;
}
